#!/bin/sh
# 
#
#
# File:     install.sh
# Purpose:  Installation script for Xerox Enhanced LPR Filters (XELF)
# Author:   
# Created:  
# License:  Copyright (C) 2002 Xerox Corporation.  All rights reserved.
#           Copyright protection claimed includes all forms and matters
#           of copyrightable material and information now allowed by
#           statutory or judicial law or hereinafter granted, including
#           without limitation, material generated from the software
#           programs which are displayed on the screen such as icons,
#           screen displays looks, etc.
#
#  Usage:  install.sh           - Installs the XELF packages stored in
#                                 the current directory.  
#          install.sh <PKGDIR>  - Installs the XELF packages stored in 
#                                 the $PKGDIR directory.
#
# Modification History:
# ==========================================================================
# Date       Author         [SP]AR    Comments
# ~~~~~~~~~~ ~~~~~~~~~~~~~~ ~~~~~~~~  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# 7/2/2002   rjager                   original install script created
# 5/10/2006  Ro				   XRXelf -> xrxelf tar file name change
# 1/10/2007  Ro				   Added platform check for release 1.5
# 1/10/2007  Ro				   Changed tar file name to s.tar.z (sparc) or 8.tar.z (x86) to avoid windows file name limitations
# 1/10/2007  Ro				   Added support to detect platform and select the appropriate tar file
# 1/18/2007  RO				   Added correction for Nuvera bad link to symbolic host name
# 5//4/2009  RO				   Added routine to change ownership on the /var/spool/XRXelf directory and files to support rel 62.90.83 						          
#
############################################################################




#===========================
# COMMON DEFINES:
#===========================
PRODUCTNAME="Xerox Enhanded LPR Filters"     # Name of product used during display
XRXUSRDIR=/export/home/xrxusr           # xrxusr home directory
USER=`/usr/ucb/whoami`                  # Login name of user executing this program
TMPDIR=/tmp                             # Temp files
TMPFILE1=$TMPDIR/tmp1
TMPFILE2=$TMPDIR/tmp2
THISAPPCAPTION="installation"           # Program's purpose: used during display
ALLARGS=$*                              # Get/store all command line arguments passed
BASE_DIR=/opt/XRXelf
VAR_DIR=/var/spool/XRXelf
TRUE="1"
FALSE="0"
host_nameame="not_defined"
bad_link_indicator="0"


#===========================
# PACKAGING DEFINES:
#===========================
#PRODPKG="xrxelf"
PRODPKG="xe"
PACKAGENAMES="XRXelf" # Name of packages that will be installed or removed
OLDPACKAGENAMES=$PACKAGENAMES # Name of packages that will be installed or removed
TARZDIR="./"                           # Location where the <PACKAGE>.tar.Z file should exist (CD Install) 
DEF_PKGDIR="/var/tmp"                  # PKGDIR used if not supplied at command line 



#===========================
# LANGUAGE SETUP DEFINES:
#===========================
LANG_SELECTED="none"                   # Language chosen


#=============================
# LICENSE BAK/RESTORE DEFINES:
#=============================
LICENSEFILE="license.dat"
LICENSEDIR="/opt/XRXelf/etc"
LICENSEBAKDIR="/var/tmp/xelf_lic_bak"
LICENSE_BACKED_UP=""


#===========================
# WORKSPACE MENU DEFINES:
#===========================
OWMENUFILE=$XRXUSRDIR/.openwin-menu    # OpenWin menu config file
CDEMENUFILE=$XRXUSRDIR/.dt/dtwmrc      # CDE menu config file



#===========================
# UNINSTALLER DEFINES:
#===========================
UNINSTALLER="uninstall.sh"              # Name of uninstaller application
UNINSTALLER_DIR="."                     # Location of the uninstaller
UNINSTALLER_ARGS="rm_package_nobanner"  # Execute without a start/end banner


#===========================
# ERROR HANDLING DEFINES:
#===========================
CURR_STEP=""                           # Step where the error occured 
ERROR_ITEM=""                          # The item that caused the error
PROGRAMABORTVALUE=2                    # Return value for when an abort execution occurs

PATH=/opt/XRXelf:/opt/XRXelf/main:/usr/sbin:/usr/bin:/usr/ucb:/sbin:/packages; export PATH


###################################
# INSTALL Enhanced LPR Filters
###################################
Start_Install()
{

 # # #
 # First get and save all arguments passed to program
 # Prepare installation by:
 # - Showing banner. 
 # - Killing all dpips processes running.
 # - Becoming root if not already.
 # Begin installation by:
 # - Getting/unpacking the new package located at PKGDIR.
 #   If package cannot be found we exit installation.
 # - Remove any previous instances of the package and related patches.
 #   This is done by an external call to uninstall.sh with args 
 #   to run quietly.
 # - Add the new package to the system.
 #
 
  get_args $ALLARGS
  
  Installation_Init
  Verify_Platform
  Install_Package
  Installation_Cleanup
  #ver_hostname
  Adjust_Existing_File_Ownership
  displayComplete
  start_GUI
}



Installation_Init()
{
  create_tmp_dir
  display_banner
  end_Xelfgui_processes
  become_root
}

ver_hostname() 
{
#verify that the symbolic link to the host name is corrupted
#lrwxrwxrwx   1 root     root          29 Feb 17 14:03 temp -> /var/spool/lp/tmp/box3

	 
	
	cd /tmp
	rm /tmp/xelf_link_info.dat
	cd /var/spool/lp
	e=`ls -l *temp`
	echo "$e" >> /tmp/xelf_link_info.dat
	bad_link_indicator=`fgrep -c "/var/spool/lp/tmp/box3" /tmp/xelf_link_info.dat`
	
	 
	if [ "$bad_link_indicator" != "0" ];
	then
		clear
		echo "\nThe symbolic link to the host name is corrupted"
		echo "\nDo you want to correct it?"
		echo "\nEnter Y to correct N to terminate."
		getYNans || abortProcess
		update_link
	fi
	
	
	

}

update_link()
{
	echo "\nUpdating the symbolic link to the host name"
	host_name=`hostname`
	cd /var/spool/lp 
	unlink temp
	ln -s /var/spool/lp/tmp/$host_name temp
	
	svcadm disable print/server
	sleep 2
	svcadm enable print/server
	sleep 2
	ls -l *temp
	
	
	
}
Verify_Platform()
{
 # PLATFORM=`isainfo -k`
 PLATFORM=`uname -p`
  if [ $PLATFORM = "sparc" ] 
  then
  	PRODPKG="s"
   	echo "The sparc version of the software will be installed."
  else
   	PRODPKG="8"
   	echo "The x86 version of the software will be installed."
   	
  fi

  VER=`uname -r`
  if [ $VER = "5.11" ] 
  then
    break
  else
    echo "The version of the operating system must be 5.11 "
    echo "This system contains version $VER."
    abortProcess
  fi
}


start_GUI()
{  
  /opt/XRXelf/sxg.sh
}


Installation_Cleanup()
{  
   cleanup_pkg_dir
}

#Adjust the file ownership for existing files
Adjust_Existing_File_Ownership()
{
	if [ -d "$VAR_DIR"  ] ; then 
		chown -R -h lp:lp /var/spool/XRXelf
		chown -R -h lp:lp /var/spool/XRXelf/*
	fi
}

###################################
# UTILITY FUNCTIONS
###################################
get_args()
{
  # $0 - Name of process (command name)
  # $1 - First arg: PKGDIR supplied by user
  
  # On first run, GOT_ARGS is not defined (!=yes)
  # We get all args passed to program
  if [ "$GOT_ARGS" != "yes" ];
  then
    # PKGDIR = arg 1 if defined else = DEF_PKGDIR
    PKGDIR=${1:-$DEF_PKGDIR}

    # Here we already got the arguments passed for installation
    # Set GOT_ARGS flag so that we don't get arguments again
    # in the case where we are rexecuting becasue we were not a root process 
    GOT_ARGS=yes

    # Update admin dir using the new pkgdir location
    ADMINDIR=$PKGDIR 

    # Export/Save the following values for the rexecution scenario 
    # else they will be lost or overwritten
    export GOT_ARGS
    export PKGDIR 
    export ADMINDIR

  fi


}


pause()
{
  sleep 2
}

create_tmp_dir()
{
  if [ -d $TMPDIR ]
  then
     break
  else
    echo Making Temp Directory $TMPDIR
    mkdir $TMPDIR
    chmod a+w $TMPDIR   # Everyone can write to it
  fi
}

beep()
{
  echo ""
}

#verify_os_level()
#{

#  VER=`uname -r`
#  if [ $VER = "5.8" ] 
#  then
#    break
#  else
#    echo "The version of the operating system must be 5.8 "
#    echo "This system contains version $VER."
#    abortProcess
#  fi

#}

abortProcess()
{
  echo "\nAborting $THISAPPCAPTION...\n"
  exit $PROGRAMABORTVALUE
}

exitProcess()
{
  exit 0
}


###################################
# GENERAL MESSAGES
###################################
display_banner()
{
 if [ "$BANNERSEEN" != "yes" ]; then
   clear
   echo "\nWelcome to the $PRODUCTNAME $THISAPPCAPTION program."
   echo "\nThis program will install or upgrade the $PRODUCTNAME\n package on the system."
   echo "\nDo you want to continue with the $THISAPPCAPTION? [y/n]: \c"
   getYNans || abortProcess
   
   # export BANNERSEEN so that we will not show this banner again
   # when we re-enter as root
   BANNERSEEN=yes; export BANNERSEEN
 fi
}

displayIncompleteSetup_WARNING()
{
  #  display Warning message: "$CURR_STEP not successfully installed."
  #  display "Refer to User guide or documentation." 
  #  prompt to continue or abort 
  beep
  beep
  echo "\n!!!!!!        WARNING        !!!!!!" 
  echo "The required $CURR_STEP was not successful."
  echo "Please refer to the $PRODUCTNAME documentation to later complete this step."
  echo "\nDo you want to continue with the installation? [y/n]: \c"
  getYNans || abortProcess
}

displayFileNotFound_ABORT_ERROR()
{
   beep
   beep
   echo "ERROR: $ERROR_ITEM not found at `pwd` location."
   echo "       Please verify that $ERROR_ITEM exists at `pwd` location"
   echo "       and rexecute $0."
   abortProcess
}  

displayComplete()
{
  echo
  echo
  echo
  echo ">> The $THISAPPCAPTION is now complete."
  echo ">>"
  #echo ">> Before trying to print jobs using the"
  #echo ">> $PRODUCTNAME, additional setup is required."
  #echo ">>"
  #echo ">> See Chapter 2 of 'Using the $PRODUCTNAME,'"
  #echo ">> which can be found on the $THISAPPCAPTION CD."
  #echo
  #echo 
#  restartPrompt
}



display_RemovePackage_message()
{
 if [ "$BANNERSEEN2" != "yes" ]; then
  beep
  echo "The $PRODUCTNAME package is already installed on the system.\n"
  echo "UPGRADING requires that the following $PRODUCTNAME" 
  echo "component(s) first be removed:"
  echo " - The $PRODUCTNAME package (application and configuration files)"
  echo
  echo "Enter y to continue with removal or n to abort $THISAPPCAPTION? \c"
  getYNans || abortProcess
  echo
  # export BANNERSEEN2 so that we will not show this banner again
  # when we re-enter as root
  BANNERSEEN2=yes; export BANNERSEEN2
 fi
 
}




###################################
# END ALL Xelf PROCESSES
###################################
end_Xelfgui_processes()
{
  # Return if we already did this step!!
  if [ "$DIDSTEP_ENDXELF_PROCESSES" = "yes" ]; then
    return;
  fi

  if [ -d "$BASE_DIR/xelfgui"  ]; then
    echo "\n>> Ending all $PRODUCTNAME services..."
    endGui  
    DIDSTEP_ENDXELF_PROCESSES=yes; export DIDSTEP_ENDXELF_PROCESSES
  fi
}


endGui()
{
  if [ -f "$BASE_DIR/xelfgui/gui.sh"  ]; then
    $BASE_DIR/xelfgui/gui.sh stop >/dev/null 2>&1  
  fi
}





###################################
# INSTALLER INTERACTION
###################################
become_root()
{
# If we are not a root process
#  Display need to become root
#  while true
#    Attempt re-execution as root (prompts for root password)
#    If return value is an error but not $PROGRAMABORTVALUE,
#    then, password entry may be unsuccessful, 
#      prompt to try password entry again: y/n
#      If y, attempt re-execution again
#      IF n, abort process       
#    If return value is not an error,
#      we will re-execute this process as root and never re-enter here again
#      when new process ends, we will gracefully exit first process

 if [ "`/usr/ucb/whoami`" != "root" ]; then

    echo "\nYou need to become root to continue $THISAPPCAPTION."

    # Allow multiple password re-entry if attempt failed
    
    while true
    do

        echo "Please enter your root password at the prompt below.\n"
	     
        # Attempt rexecution as root
        su root -c "exec ./$0"
        
        # Check for errors returned when attempting password entry
        # 0= successfull, 1 or any other= errors, $PROGRAMABORTVALUE= value used when aborting, 
        # If return value is an error but not $PROGRAMABORTVALUE,
        # then, password entry may be unsuccessful,
        #    prompt to try password entry again
        # Else, exit gracefully

        if [ $? != "0"  -a  $? != "$PROGRAMABORTVALUE" ]
        then
           echo "Would you like to try again? [y/n] \c"
           getYNans || abortProcess
        else
           exitProcess
        fi
        
   done
 fi


}

getYNans()
{
  while true
  do
	read response
	case $response in
	 [Yy]*) return 0 ;;
	 [Nn]*) return 1 ;;
	  *) echo "Please answer y or n: \c" ;;
        esac
    done
}

restartPrompt()
{
  # Prompt to restart
  # If N, exit
  # Else if Y, init 6

  echo ">> Would you like to reboot to allow changes to take effect? [y/n]\c"
  getYNans || exitProcess
  
  init 6
}



###################################
# PACKAGE UPDATES
###################################
add_new_package()
{ 
  echo "\n\n>> Adding $PRODUCTNAME packages to system...\n"
  verify_pkg_dir_exist
  verify_package_exist

  for pkgnm in $PACKAGENAMES
  do
        echo $pkgnm
        pkgadd -d $PKGDIR -a $PKGDIR/admin $pkgnm
  done

  
  echo "\nThe $PRODUCTNAME package was successfully added.\n" 
  pause
}


remove_old_package()
{
  # Check if packages already installed
  #  If package exist: Prompt do you want to remove pkgs for install
  #    If no, exit
  #    Else, continue

  echo "\n>> Checking for previously installed packages...\n"
  RMPKGS=n
  for pkgnm in $OLDPACKAGENAMES
  do
     pkginfo | grep $pkgnm 1>/dev/null
     if [ $? -eq 0 ]; then
        RMPKGS=y
        display_RemovePackage_message
        break;      
     fi
  done

  if [ "$RMPKGS" != "y" ];
  then
     echo "\nNo previous packages installed. "
  else
     backup_license

     echo "\n>> Removing previous packages..."
     verify_uninstaller_exist
     $UNINSTALLER_DIR/$UNINSTALLER $UNINSTALLER_ARGS
     if [ "$?" != "0" ];
     then
       echo "\nPackage removal failed, process aborting."
       abort_process
     else
       echo "\nPrevious packages successfully removed.\n"
       pause
     fi
  fi   

  pause
}


get_new_package()
{

 # PKGDIR stores the root directory of the PRODPKG
 # e.g. PKGDIR="/var/tmp" (DEF_PKGDIR)
 #      PKGDIR="/packages/XRXelf1.0v1"
 #      PKGDIR="XRXelf1.0v1"
 #
 # If PKGDIR not equal to the DEF_PKGDIR,  (OTHER INSTALL)
 #    The user supplied a new PKGDIR location for install
 #    Unpack from the PKGDIR supplied
 # Else,
 #    New PKGDIR not supplied
 #    Install from current location (e.g. CDROM INSTALL)
 #    Unpack the PRODPKG.tar.Z file to PKGDIR
 #

 if [ $PKGDIR != $DEF_PKGDIR ]
 then
   unpack_Package_newdir    
 else  
    unpack_TARZ_file
 fi

}

install_pkg_cleanup()
{
 # Remove the package copied to the DEF_PKGDIR
 if [ $PKGDIR = $DEF_PKGDIR ]
 then
  rm -rf $PKGDIR/$PRODPKG
 fi

# We will restore the license file that may have been 
# backed up.
restore_license
}



Install_Package()
{
  echo "\n>> Package Installation in progress..."
  get_new_package
  remove_old_package
  add_new_package
  install_pkg_cleanup
}



verify_admin_file_exist()
{
 # Proper execution of this program requires that 
 # "our" admin file exist.
 # Package installation can occur without "our" admin file, 
 # because pkgadd will use it's own default.
 #
 # If this occurs, installation is unpredictable.
 #
 # REQUIREMENTS:
 # admin file location = "./" or $ADMINDIR

 # First, verify that admin file exists at ADMINDIR
 
 if [ -f $ADMINDIR/admin ]
 then
   return;
 fi

 # If admin file not at ADMINDIR
 # check at "./" location
 if [ -f "./admin" ];
 then
   ADMINDIR="."
 fi

}

verify_pkg_dir_exist()
{
 # If PKGDIR do not exist,
 #    create it.
 #
 if [ -d $PKGDIR ]
 then
   break
 else
   echo Making Directory $PKGDIR
   mkdir $PKGDIR
 fi
 
}

empty_pkg_dir_for_extraction()
{
  cleanup_pkg_dir
}


cleanup_pkg_dir()
{

 # If $PKGDIR/$PRODPKG exists,
 #    remove it
 #
 # This removes any PRODPKG instances that may 
 # have been left around after a previous install

 if [ -d $PKGDIR/$PRODPKG ]
 then
   # Remove $PKGDIR/$PRODPKG dir
   rm -rf $PKGDIR/$PRODPKG
 fi


 # Also, remove these pkg directories
 for pkgnm in $OLDPACKAGENAMES
 do
     if [ -d $PKGDIR/$pkgnm ]
     then
       # Remove $PKGDIR/$pkgnm dir
       rm -rf $PKGDIR/$pkgnm
     fi
 done

 # delete admin file if found
 rm -f "$PKGDIR/admin" >/dev/null 2>&1
 
}


verify_package_exist()
{
 # Verify package existance at PKGDIR
 
 if [ -d $PKGDIR/XRXelf ] 
 then
   break;
 else
   ERROR_ITEM="\" $PRODUCTNAME \" packages"
   displayFileNotFound_ABORT_ERROR
 fi
}

 
verify_tarz_package_exist()
{
 # If tar.Z file exists,
 #    unpack it.
 # else,
 #   Display ERROR:  Product Package file not found.
 #   Abort Installation
 #
 #echo "\n>> Searching for the $PRODPKG.tar.Z file at `pwd` location...\n"
 #if [ -r $PRODPKG.tar.Z ]
 if [ -r $PRODPKG.tar.z ]
 then
   break;
   #echo "$PRODPKG.tar.Z file found.";
 else
   #ERROR_ITEM=\"$PRODPKG.tar.Z\"
   ERROR_ITEM=\"$PRODPKG.tar.z\"
   displayFileNotFound_ABORT_ERROR
 fi
}


verify_uninstaller_exist()
{
 if [ -f $UNINSTALLER_DIR/$UNINSTALLER ]
 then
   break;
 else
   ERROR_ITEM=\"$UNINSTALLER_DIR/$UNINSTALLER\"
   displayFileNotFound_ABORT_ERROR
 fi
}


unpack_TARZ_file()
{
 # At this step,
 # PKGDIR value equals DEF_PKGDIR.
 # TARZLOC value equals the location of the .tar.Z file
 # If PRODPKG.tar.Z (packed) file exist at TARZLOC location.

 # Verify that PRODPKG.tar.Z file and PKGDIR exists
 # before unpacking tar.Z file.

 verify_tarz_package_exist

 verify_pkg_dir_exist
 
 empty_pkg_dir_for_extraction

 echo "\n>> Extracting `pwd`/$PRODPKG.tar.Z file...\n"
 
 # Copy file to PKGDIR for extraction
 #cp $TARZDIR/$PRODPKG.tar.Z $PKGDIR/$PRODPKG.tar.Z
 cp $TARZDIR/$PRODPKG.tar.z $PKGDIR/$PRODPKG.tar.Z 

 chmod a+rw $PKGDIR/$PRODPKG.tar.Z
 
 # Save the last location before PKGDIR was entered
 lastlocation=`pwd`
 cd $PKGDIR
 # Unpack file
 uncompress $PRODPKG.tar.Z
 tar xfo $PRODPKG.tar
 rm $PRODPKG.tar

 # Go back to the last location 
 cd $lastlocation
 echo "Extraction complete." 

 # Unpacking of *.tar.Z file complete
}



unpack_Package_newdir()
{ 
 
 # PKGDIR value is supplied by user.
 # PRODPKG if exist, will already be unpacked.

 # At this step we only verify that the package exists.
 #
 # If PKGDIR/PRODPKG directory exists before proceeding
 #     echo Installing package

 verify_pkg_dir_exist
 verify_package_exist
 verify_admin_file_exist
 
}


backup_license()
{
 if [ -f "$LICENSEDIR/$LICENSEFILE" ]
 then 
   echo "\n>> License backup in progress...\n"

   if [ -d "$LICENSEBAKDIR" ]
   then 
     rm -rf "$LICENSEBAKDIR"
   fi

   mkdir "$LICENSEBAKDIR"

   # Copy license file to backup directory
   cp -p "$LICENSEDIR/$LICENSEFILE" "$LICENSEBAKDIR/"

   if [ "$?" = "0" ]
   then 
     LICENSE_BACKED_UP="$TRUE"
     echo "License successfully backed up.\n"
   fi

 fi

}


restore_license()
{

 if [ "$LICENSE_BACKED_UP" = "$TRUE" ]
 then 
   echo "\n>> License restore in progress...\n"

   if [ -f "$LICENSEBAKDIR/$LICENSEFILE" ]
   then 
      # If directory structure exist after pkg install,
      #   restore license
      if [ -d "$LICENSEDIR" ]
      then 
         # Restore license file to the proper location
         cp -p "$LICENSEBAKDIR/$LICENSEFILE" "$LICENSEDIR"
         if [ "$?" = "0" ]
         then 
           echo "License successfully restored."
           # Reset flag
           LICENSE_BACKED_UP="$FALSE"

           # Cleanup
           rm -rf $LICENSEBAKDIR
	   return;
         fi

      fi
   fi

   echo "Error: License unsuccessfully restored."
 fi

}

###################################
# BEGIN EXECUTION HERE
###################################
Start_Install $ALLARGS

